@@ -4,6 +4,7 @@ Utility for forward contracts
 import re
 from calendar import month_abbr
 
+import numpy as np
 import pandas as pd
 
 from commodutil import dates
@@ -705,6 +706,13 @@ def spread_combination(contracts, combination_type, verbose_columns=True):
                 columns={x: colmap[x] for x in c_contracts.columns}
             )
         return c_contracts
+    if combination_type == "half year":
+        c_contracts = half_year_contracts(contracts)
+        return c_contracts
+    if combination_type == "half year spread":
+        c_contracts = half_year_spreads(half_year_contracts(contracts))
+        return c_contracts
+
     if combination_type.startswith("q"):
         q_contracts = quarterly_contracts(contracts)
         m = re.search("q\dq\dq\d", combination_type)
@@ -794,8 +802,112 @@ def spread_combination(contracts, combination_type, verbose_columns=True):
             return c
 
 
-if __name__ == "__main__":
-    from pylim import lim
+def reject_outliers(data, m=2):
+    return data[abs(data - np.mean(data)) < m * np.std(data)]
 
-    df = lim.series(["CL_2023Z", "CL_2024F"])
-    spread_combination(df, "DecJan")
+
+def extract_expiry_date(contract, expiry_dates):
+    if expiry_dates:
+        return expiry_dates.get(contract, contract + pd.offsets.MonthEnd(1))
+
+    return contract + pd.offsets.MonthEnd(1)
+
+
+def determine_roll_date(df, expiry_date, roll_days):
+    cdf = df.copy().dropna(how="all", axis="rows") # remove non-trading days
+    if expiry_date in cdf.index:
+        idx_position = cdf.index.get_loc(expiry_date)
+        new_idx_position = idx_position - roll_days
+
+        if new_idx_position >= 0:
+            return cdf.index[new_idx_position]
+
+    return expiry_date
+
+
+def continuous_futures(df, expiry_dates=None, roll_days=0, front_month=1, back_adjust=False) -> pd.DataFrame:
+    """
+    Create a continuous future from individual contracts by stitching together contracts after they expire
+    with an option for back-adjustment.
+
+    :param df: DataFrame with individual contracts as columns.
+    :param expiry_dates: Dictionary mapping contract dates to their respective expiry dates.
+    :param roll_days: Number of days before the expiry date to roll to the next contract.
+    :param front_month: Determines which contract month(s) to select. Can be an int or list of ints.
+    :param back_adjust: If True, apply back-adjustment to the prices.
+    :return: DataFrame representing the continuous future for each front month.
+    """
+    if isinstance(front_month, int):
+        front_month = [front_month]  # convert to list if it's a single integer
+
+    df.columns = [pd.to_datetime(x) for x in df.columns]
+
+    # Format expiry_dates if provided
+    if expiry_dates:
+        expiry_dates = {
+            pd.to_datetime(x): pd.to_datetime(expiry_dates[x]) for x in expiry_dates
+        }
+
+    continuous_dfs = []
+
+    for front_month_x in front_month:
+        mask_switch = pd.DataFrame(index=df.index, columns=df.columns)
+        mask_adjust = pd.DataFrame(index=df.index, columns=df.columns)
+
+        # Iterating over the columns (contracts)
+        for contract in df.columns:
+            prev_contract = contract - pd.offsets.MonthBegin(1)
+            next_contract = contract + pd.offsets.MonthBegin(1)
+
+            # Determine expiry date for each contract
+            expiry_date = extract_expiry_date(contract, expiry_dates)
+            prev_contract_expiry_date = extract_expiry_date(prev_contract, expiry_dates)
+
+            # Adjust expiry date based on roll_days
+            roll_date = determine_roll_date(df, expiry_date, roll_days)
+            prev_contract_roll_date = determine_roll_date(df, prev_contract_expiry_date, roll_days)
+
+            # Set the cells to 1 where the index date is between the current contract date and the adjusted expiry date
+            mask_switch.loc[
+                (mask_switch.index > pd.Timestamp(prev_contract_roll_date))
+                & (mask_switch.index <= pd.Timestamp(roll_date)),
+                contract,
+            ] = 1
+
+            # Keep a track of difference between front and back contract on roll date
+            if roll_date in df.index and contract in df.columns and next_contract in df.columns:
+                adj_value = df.at[roll_date, next_contract] - df.at[roll_date, contract]
+                mask_adjust.loc[
+                    (mask_switch.index > prev_contract_roll_date)
+                    & (mask_switch.index <= roll_date),
+                    contract,
+                ] = adj_value
+
+        mask_switch = mask_switch.shift(front_month_x - 1, axis=1)  # handle front month eg M2, M3 etc
+        # Multiply df with mask and sum along the rows
+        continuous_df = df.mul(mask_switch, axis=1).sum(axis=1, skipna=True, min_count=1)
+        continuous_df = pd.DataFrame(continuous_df, columns=[f"M{front_month_x}"])
+
+        # Back-adjustment
+        if back_adjust:
+            mask_adjust_series = mask_adjust.fillna(method='bfill').sum(axis=1, skipna=True, min_count=1).fillna(0)
+            continuous_df = continuous_df.add(mask_adjust_series, axis=0)
+
+        continuous_dfs.append(continuous_df)
+
+    # Concatenate all dataframes for each front month
+    final_df = pd.concat(continuous_dfs, axis=1).dropna(how="all", axis="rows")
+
+    # Store mask in attributes for reference
+    final_df.attrs["mask_switch"] = mask_switch
+    final_df.attrs["mask_adjust"] = mask_adjust
+
+    return final_df
+
+
+
+# if __name__ == "__main__":
+      # from pylim import lim
+#
+#     df = lim.series(["CL_2023Z", "CL_2024F"])
+#     spread_combination(df, "DecJan")
