@@ -86,3 +86,34 @@ def mergem(c):
         c,
     )
     return c
+
+
+def generate_lambda(expression, columns):
+    """
+    Convert a simple expression string into a lambda function suitable for DataFrame operations.
+    """
+    if isinstance(columns, pd.MultiIndex):
+        columns = columns.levels[0]
+
+    # Sort columns by length in descending order to handle substring issues
+    columns = sorted(columns, key=len, reverse=True)
+
+    for col in columns:
+        # Use regular expressions to replace whole word matches
+        expression = re.sub(rf"\b{re.escape(col)}\b", f"x['{col}']", expression)
+
+    return lambda x: eval(expression)
+
+
+def apply_formula(df: pd.DataFrame, formula: str) -> pd.DataFrame:
+    """
+    Given a dataframe apply the formula to the columns in the dataframe
+    """
+
+    formula_l = generate_lambda(formula, df.columns)
+    dfr = df.apply(formula_l, axis=1)
+
+    if isinstance(dfr, pd.Series):
+        dfr = pd.DataFrame(dfr, columns=[formula])
+
+    return dfr
